require 'SmartModeling\Utilities.rb'
require 'SmartModeling\vectorTools.rb'
require 'SmartModeling\edgeTools.rb'

def is2EdgesMakeAFace(e1,e2)
  line1=[e1.start.position,e1.end.position]
  line2=[e2.start.position,e2.end.position]
  va=e1.end.position-e1.start.position
  vb=e2.end.position-e2.start.position
  pt=Geom.intersect_line_line(line1,line2)
  if pt==nil
    if (va.parallel? vb)&&!is2EdgesInOneLine(e1,e2)
      vb=e1.end.position-e2.start.position
      vc=va.cross vb
    else
      pt
    end
  else
    vc=va.cross vb
    vc
  end
end

def is2EdgesInOneLine(e1,e2)
  isinoneline=true
  va=e1.end.position-e1.start.position
  vb=e2.end.position-e2.start.position
  if va.parallel? vb
    vc=e2.start.position-e1.start.position
    vc=e2.end.position-e1.start.position if vc.length==0
    isinoneline=false if !(va.parallel? vc)
  else
    isinoneline=false
  end
  isinoneline
end

def isEdgeOnPanle(e,p)
  isonface=true
  line=[e.start.position,e.end.position]
  pt=Geom.intersect_line_plane(line, p)
  isonface=false if pt!=nil
  isonface
end

def vecOneLine(e)
  vr=Geom::Vector3d.new(0,-1,0)
  vz=Geom::Vector3d.new(0,0,1)
  ve=e.end.position-e.start.position
  if !(ve.parallel? vz)
    vz=ve.cross vz
    vr=vz.cross ve
  end
  vr
end

def edge2face(width,h,type)
  vec_z=Geom::Vector3d.new 0,0,1
  po=Geom::Point3d.new 0,0,0
  model=Sketchup.active_model
  ents=model.active_entities
  entsel=model.selection
  
  return if entsel.length==0
    
  gp0=ents.add_group
  gp0ent=gp0.entities
  
  r2d=180/Math::PI
  
  a_edge=[]
  entsel.each{|i| a_edge.push(i) if i.typename=="Edge"}
  return if a_edge.length==0
  
  if a_edge.length==1
    vec_z=vecOneLine(a_edge[0])
  else
    e0=a_edge[0]
    makeplane=false
    vt=nil
    1.upto(a_edge.length-1) do |i|
      vt=is2EdgesMakeAFace(e0,a_edge[i])
      if vt!=nil
        makeplane=true
        break
      end
    end
    if makeplane
      p=[e0.start.position, vt]
      alllineinpanel=true
      a_edge.each {|i| alllineinpanel=false if isEdgeOnPanle(i,p)==false}
      if alllineinpanel
        vec_z=vt
      else
        UI::messagebox "Straight lines are not coplanar"
        return
      end
    else
      alllineinoneline=true
      1.upto(a_edge.length-1){|i| alllineinoneline=false if is2EdgesInOneLine(e0,a_edge[i])==false}
      if alllineinoneline
        vec_z=vecOneLine(a_edge[0])
      else
        UI::messagebox "Straight lines are not coplanar"
        return
      end
    end
  end

  a_vertex=[]
  a_edge.each do |i|
    a_vertex.push(i.vertices[0]) if not a_vertex.include? i.vertices[0]
    a_vertex.push(i.vertices[1]) if not a_vertex.include? i.vertices[1]
  end
  
  a_vertexIndex=[]
  a_vertexBetween=[] 
  
  a_vertex.each do |i|
    a_attchedEdges=i.edges
    avilable_edges=[]
    vec_edges=[]
    a_attchedEdges.each{|j| avilable_edges.push(j) if a_edge.include? j}
    
    avilable_edges.each do |j|
      p_start=i.position
      p_end=(j.other_vertex i).position
      vec_j=p_start.vector_to p_end
      vec_edges.push(vec_j)          
    end
    vec_edges=sortVectors(vec_edges,vec_z)
    a_vertexIndex.push(vec_edges)
    
    vec_edges.push(vec_edges[0])
    a_t=[] 
    0.upto vec_edges.length-2 do |j|
      a_t.push([vec_edges[j],vec_edges[j+1]])
    end
    
    a_b=[]
    if avilable_edges.length!=1
      0.upto a_t.length-1 do |j|
        v_t=divideVector(a_t[j][0],a_t[j][1],50,vec_z)
        v_t.length=width/Math::sin(rightAngleBetween(a_t[j][0],a_t[j][1],vec_z)/2)
        a_b.push(v_t)
      end
    else
      vrmin=a_t[0][0]*vec_z
      vrmin.length=width
      vrmax=vrmin.reverse
      a_b.push(vrmin,vrmax)
    end
    a_vertexBetween.push(a_b)
  end
 
  a_edge.each do |i|
    pointBuffer=[]
    i.vertices.each do |j|
      index=a_vertex.index j
      v_edge=j.position.vector_to (i.other_vertex j)
      v_vertex=po.vector_to j.position      
      vmin=rightAngleMin(v_edge,a_vertexBetween[index],vec_z)+v_vertex
      pmin=Geom::Point3d.new vmin.x,vmin.y,vmin.z
      pointBuffer.push(pmin)
      pointBuffer.push(j.position)
      vmax=rightAngleMax(v_edge,a_vertexBetween[index],vec_z)+v_vertex
      pmax=Geom::Point3d.new vmax.x,vmax.y,vmax.z
      pointBuffer.push(pmax)
    end
    gp=gp0ent.add_group
    gpents=gp.entities
    gpents.add_face pointBuffer
    gp.explode
  end
  margeFaces(gp0ent)
  rface=[]
  gp0ent.each{|i| rface.push(i) if i.typename=="Face"}
  rface.each{|i| i.pushpull -h,false if h!=0}
  margeFaces(gp0ent)
end